﻿using System;
using System.Windows;
using GalaSoft.MvvmLight.Threading;
using AnimationCreator.ViewModel;

namespace AnimationCreator
{
	public partial class App : Application
	{
		public App()
		{
			Startup += Application_Startup;
			Exit += Application_Exit;
			UnhandledException += Application_UnhandledException;

			InitializeComponent();
		}

		private void Application_Startup(object sender, StartupEventArgs e)
		{
			RootVisual = new MainPage();
			DispatcherHelper.Initialize();

			if (IsRunningOutOfBrowser)
			{
				CheckAndDownloadUpdateCompleted += new CheckAndDownloadUpdateCompletedEventHandler(App_CheckAndDownloadUpdateCompleted);
				CheckAndDownloadUpdateAsync();
			}
		}

		private void App_CheckAndDownloadUpdateCompleted(object sender, CheckAndDownloadUpdateCompletedEventArgs e)
		{
			if (e.Error != null)
			{
				MessageBox.Show("An error occurred while checking for an update: " + e.Error.Message,
						"Animation Texture Creator",
						MessageBoxButton.OK);
			}
			else
			{
				if (e.UpdateAvailable)
				{
					MessageBox.Show("An update of this application has been downloaded and installed." + Environment.NewLine
						+ "Please restart Animation Texture Creator to apply the update.",
						"Animation Texture Creator",
						MessageBoxButton.OK);
				}
			}
		}

		private void Application_Exit(object sender, EventArgs e)
		{
			ViewModelLocator.Cleanup();
		}

		private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
		{
			// If the app is running outside of the debugger then report the exception using
			// the browser's exception mechanism. On IE this will display it a yellow alert 
			// icon in the status bar and Firefox will display a script error.
			if (!System.Diagnostics.Debugger.IsAttached)
			{

				// NOTE: This will allow the application to continue running after an exception has been thrown
				// but not handled. 
				// For production applications this error handling should be replaced with something that will 
				// report the error to the website and stop the application.
				e.Handled = true;
				Deployment.Current.Dispatcher.BeginInvoke(delegate
				{
					ReportErrorToDOM(e);
				});
			}
		}
		private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
		{
			try
			{
				string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
				errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

				System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight 2 Application " + errorMsg + "\");");
			}
			catch (Exception)
			{
			}
		}
	}
}
